#!/bin/perl

#----------------------------------------------------------
# dxf2txt
#
# Extracting TEXT/MTEXT/DIMENSION/ATTDEF blocks from DXF file
# and creating file with text blocks for translation
# with CAT systems.
#----------------------------------------------------------
# Method:
#   1. Find TEXT/MTEXT/DIMENSION/ATTDEF header in DXF
#   2. Skip attribute lines
#   3. Collect text
# Restriction:
#   Not suitable for Autocad 2008 DXF format.
#----------------------------------------------------------
# Command line parameters:
#   -----------------------------------
#   Parameters        | Descriptions
#   -----------------------------------
#   dxf-file name(s)  | Name(s) of DXF file to process
#   (wildcards enabled)
#
# Input files:
#   DXF-file(s)
#
# Output files:
#   TXT file(s) with TEXT/MTEXT/DIMENSION/ATTDEF blocks. Format:
#
#      {1-nnn}
#      <Text line from MTEXT block>
#      {2-nnn}
#      <Text line from TEXT block>
#      {3-nnn}
#      <Text line from DIMENSION block>
#      {4-nnn}
#      <Text line from ATTDEF block>
#
#   nnn is the ordinal number of TEXT/MTEXT/DIMENSION/ATTDEF block.
#   (TEXT/MTEXT/DIMENSION/ATTDEF block sequences are enumerated separately).
#----------------------------------------------------------
# (C) Copyright 2006-2009, Mikegor.
#
# Version |    Date     | Comment
#---------+-------------+--------------------
#   1.0   | 16 Nov 2006 | Original code.
#   1.1   | 25 Dec 2006 | Processing DIMENSION blocks.
#   2.0   | 26 Mar 2008 | Block headers consist of numbers only,
#         |             | multiple files processing enabled.
#   2.1   | 09 Apr 2008 | Processing of ATTDEF blocks.
#   2.2.  | 13 Sep 2009 | Cyrillic Unicode symbols support.
#----------------------------------------------------------

#---------------------------------------------------
#--- 0. Initialization
#---------------------------------------------------

#--- 0.1. Printing programm version

print "Text extractor for DXF files. Version 2.2, (c) Mikegor, 13 Sep 2009.\n";


#--- 0.2. Getting number of arguments

$narg = $#ARGV + 1;


#--- 0.3. Prining help message if no arguments

if ($narg == 0)
{
print "Usage:\n";
print "dxf2txt file1[.dxf] file2[.dxf] ...\n";
exit
}


#--- 0.4. Loop over arguments

for ($iarg=0; $iarg <= $#ARGV; $iarg++)
{


#---------------------------------------------------
#--- 1. Opening files
#---------------------------------------------------

#--- 1.1. Input dxf file


$dxf = $ARGV[$iarg];

if (! ($dxf =~ /\.dxf$/))
{
   $dxf .= ".dxf";
}

print "Opening $dxf\n";

open (DXF, '<', $dxf) or die "Cannot open '$dxf'\n";

#--- 1.2. Output txt file

$txt = $ARGV[$iarg];

if ($txt =~ /\.dxf$/)
{
   $txt =~ s/\.dxf$/.txt/;
}
else
{
   $txt .= ".txt";
}

print "Opening $txt\n";
open (TXT, '>', $txt) or die "Cannot open '$txt'\n";

#---------------------------------------------------
#--- 2. Parsing file
#---------------------------------------------------

#--- 2.0. Initialization

$i       = 0;
$jmtxt   = 0;
$jtxt    = 0;
$jdim    = 0;
$jatt    = 0;
$txtline = "";


sub procline {
   my $s = shift (@_);
   $s =~ s/\\U\+00C4//ig;
   $s =~ s/\\U\+00E4//ig;
   $s =~ s/\\U\+00DF//ig;
   $s =~ s/\\U\+00DC//ig;
   $s =~ s/\\U\+00FC//ig;
   $s =~ s/\\U\+00D6//ig;
   $s =~ s/\\U\+00F6//ig;
   $s =~ s/\\U\+00E0//ig;
   $s =~ s/\\U\+00E1//ig;
   $s =~ s/\\U\+00E8//ig;
   $s =~ s/\\U\+00E9//ig;
   $s =~ s/\\U\+00E7//ig;
   $s =~ s/\\U\+00E2//ig;
   $s =~ s/\\U\+00EA//ig;
   $s =~ s/\\U\+00EB//ig;
   $s =~ s/\\U\+00EC//ig;
   $s =~ s/\\U\+00ED//ig;
   $s =~ s/\\U\+00EE//ig;
   $s =~ s/\\U\+00E7//ig;
   $s =~ s/\\U\+00F2//ig;
   $s =~ s/\\U\+00F3//ig;
   $s =~ s/\\U\+00F4//ig;
   $s =~ s/\\U\+00F9//ig;
   $s =~ s/\\U\+00FA//ig;
   $s =~ s/\\U\+00FB//ig;
   $s =~ s/\\U\+0410//ig;
   $s =~ s/\\U\+0430//ig;
   $s =~ s/\\U\+0411//ig;
   $s =~ s/\\U\+0431//ig;
   $s =~ s/\\U\+0412//ig;
   $s =~ s/\\U\+0432//ig;
   $s =~ s/\\U\+0413//ig;
   $s =~ s/\\U\+0433//ig;
   $s =~ s/\\U\+0414//ig;
   $s =~ s/\\U\+0434//ig;
   $s =~ s/\\U\+0415//ig;
   $s =~ s/\\U\+0435//ig;
   $s =~ s/\\U\+0401//ig;
   $s =~ s/\\U\+0451//ig;
   $s =~ s/\\U\+0416//ig;
   $s =~ s/\\U\+0436//ig;
   $s =~ s/\\U\+0417//ig;
   $s =~ s/\\U\+0437//ig;
   $s =~ s/\\U\+0418//ig;
   $s =~ s/\\U\+0438//ig;
   $s =~ s/\\U\+0419//ig;
   $s =~ s/\\U\+0439//ig;
   $s =~ s/\\U\+041A//ig;
   $s =~ s/\\U\+043A//ig;
   $s =~ s/\\U\+041B//ig;
   $s =~ s/\\U\+043B//ig;
   $s =~ s/\\U\+041C//ig;
   $s =~ s/\\U\+043C//ig;
   $s =~ s/\\U\+041D//ig;
   $s =~ s/\\U\+043D//ig;
   $s =~ s/\\U\+041E//ig;
   $s =~ s/\\U\+043E//ig;
   $s =~ s/\\U\+041F//ig;
   $s =~ s/\\U\+043F//ig;
   $s =~ s/\\U\+0420//ig;
   $s =~ s/\\U\+0440//ig;
   $s =~ s/\\U\+0421//ig;
   $s =~ s/\\U\+0441//ig;
   $s =~ s/\\U\+0422//ig;
   $s =~ s/\\U\+0442//ig;
   $s =~ s/\\U\+0423//ig;
   $s =~ s/\\U\+0443//ig;
   $s =~ s/\\U\+0424//ig;
   $s =~ s/\\U\+0444//ig;
   $s =~ s/\\U\+0425//ig;
   $s =~ s/\\U\+0445//ig;
   $s =~ s/\\U\+0426//ig;
   $s =~ s/\\U\+0446//ig;
   $s =~ s/\\U\+0427//ig;
   $s =~ s/\\U\+0447//ig;
   $s =~ s/\\U\+0428//ig;
   $s =~ s/\\U\+0448//ig;
   $s =~ s/\\U\+0429//ig;
   $s =~ s/\\U\+0449//ig;
   $s =~ s/\\U\+042A//ig;
   $s =~ s/\\U\+044A//ig;
   $s =~ s/\\U\+042B//ig;
   $s =~ s/\\U\+044B//ig;
   $s =~ s/\\U\+042C//ig;
   $s =~ s/\\U\+044C//ig;
   $s =~ s/\\U\+042D//ig;
   $s =~ s/\\U\+044D//ig;
   $s =~ s/\\U\+042E//ig;
   $s =~ s/\\U\+044E//ig;
   $s =~ s/\\U\+042F//ig;
   $s =~ s/\\U\+044F//ig;
   $s =~ s/\\U\+0402//ig;
   $s =~ s/\\U\+0452//ig;
   $s =~ s/\\U\+0403//ig;
   $s =~ s/\\U\+0453//ig;
   $s =~ s/\\U\+0404//ig;
   $s =~ s/\\U\+0454//ig;
   $s =~ s/\\U\+0405//ig;
   $s =~ s/\\U\+0455//ig;
   $s =~ s/\\U\+0406//ig;
   $s =~ s/\\U\+0456//ig;
   $s =~ s/\\U\+0407//ig;
   $s =~ s/\\U\+0457//ig;
   $s =~ s/\\U\+0408//ig;
   $s =~ s/\\U\+0458//ig;
   $s =~ s/\\U\+0409//ig;
   $s =~ s/\\U\+0459//ig;
   $s =~ s/\\U\+040A//ig;
   $s =~ s/\\U\+045A//ig;
   $s =~ s/\\U\+040B//ig;
   $s =~ s/\\U\+045B//ig;
   $s =~ s/\\U\+040C//ig;
   $s =~ s/\\U\+045C//ig;
   $s =~ s/\\U\+040E//ig;
   $s =~ s/\\U\+045E//ig;
   $s =~ s/\\U\+040F//ig;
   $s =~ s/\\U\+045F//ig;
   $s =~ s/\\U\+0490//ig;
   $s =~ s/\\U\+0491//ig;
   return $s;
}

#--- 2.1. Reading file

while ($s = <DXF>)
{
   
   #--- 2.1.1. Removing EOL
   
   ++$i;
   $s =~ s/[\r\n]*$//g;

   #--- 2.1.2. Processing MTEXT block

   if ($s =~ /^ *AcDbMText *$/)
   {

      #--- 2.1.2.1. Skipping attribute lines

      for($k = 1; $k <= 14; $k++)
      {
         $s = <DXF>;
         $s =~ s/[\r\n]*$//g;
         ++$i;
      }

      #--- 2.1.2.2. Collecting text

      do
      {
         $id = <DXF>;
         $id =~ s/[\r\n]*$//g;
         ++$i;
         if (($id != 3) && ($id != 1))
         {
            die "Stop here: something is wrong in line $i\n";
         }
         $txtline .= <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;
      } until $id == 1;

      #--- 2.1.2.3. Writing text

      ++$jmtxt;

      $txtline = procline($txtline);
      print TXT "{1-$jmtxt}\n";
      print TXT "$txtline\n";
      $txtline = "";
   }

   #--- 2.1.3. Processing TEXT block

   if ($s =~ /^ *AcDbText *$/)
   {
      $s = <DXF>;
      $s =~ s/[\r\n]*$//g;
      ++$i;
      
      if ($s == 10)
      {
         for($k = 1; $k <= 8; $k++)
         {
            $s = <DXF>;
            $s =~ s/[\r\n]*$//g;
            ++$i;
         }
         $txtline = <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;
         if ($txtline !~ /^ *$/)
         {
            ++$jtxt;
            $txtline = procline($txtline);
            print TXT "{2-$jtxt}\n";
            print TXT "$txtline\n";
            $txtline = "";
         }
      }

   }

   #--- 2.1.4. Processing DIMENSION block

   if ($s =~ /^ *AcDbDimension *$/)
   {
      $s = <DXF>;
      $s =~ s/[\r\n]*$//g;
      ++$i;
      
      if ($s == 2)
      {
         for($k = 1; $k <= 16; $k++)
         {
            $s = <DXF>;
            $s =~ s/[\r\n]*$//g;
            ++$i;
         }
         $txtline = <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;
         if ($txtline !~ /^ *$/)
         {
            ++$jdim;
            $txtline = procline($txtline);
            print TXT "{3-$jdim}\n";
            print TXT "$txtline\n";
            $txtline = "";
         }
      }

   }

   #--- 2.1.5. Processing ATTDEF block

   if ($s =~ /^ *AcDbAttributeDefinition *$/)
   {
      $s = <DXF>;
      $s =~ s/[\r\n]*$//g;
      ++$i;
      
      if ($s == 3)
      {
         $txtline = <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;

         ++$jatt;
         $txtline = procline($txtline);
         print TXT "{4-$jatt}\n";
         print TXT "$txtline\n";
         $txtline = "";

         $s = <DXF>;
         $s =~ s/[\r\n]*$//g;
         ++$i;
         $txtline = <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;

         $txtline = procline($txtline);
         print TXT "{4-$jatt}\n";
         print TXT "$txtline\n";
         $txtline = "";
      }

   }

}

print "Statistics:\n";
print "$i lines parsed\n";
print "$jmtxt MTEXT blocks saved\n";
print "$jtxt TEXT blocks saved\n";
print "$jdim DIMENSION blocks saved\n";
print "$jatt ATTDEF blocks saved\n";

#---------------------------------------------------
#--- 3. Closing files
#---------------------------------------------------

close DXF;
close TXT;

#---------------------------------------------------
#--- 4. Finalization
#---------------------------------------------------

#--- 4.1. Closing main loop

}

